@@ -3,7 +3,7 @@
 *                       T e x t   S e a r c h   D i a l o g                     *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2000,2006 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXSearchDialog.cpp,v 1.29 2005/01/16 16:06:07 fox Exp $                  *
+* $Id: FXSearchDialog.cpp,v 1.33 2006/01/22 17:58:41 fox Exp $                  *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -68,7 +68,7 @@ FXIMPLEMENT(FXSearchDialog,FXReplaceDialog,NULL,0)
 // File Open Dialog
 FXSearchDialog::FXSearchDialog(FXWindow* owner,const FXString& caption,FXIcon* ic,FXuint opts,FXint x,FXint y,FXint w,FXint h):
   FXReplaceDialog(owner,caption,ic,opts,x,y,w,h){
-  accept->setText("&Search");
+  accept->setText(tr("&Search"));
   every->hide();
   replacelabel->hide();
   replacebox->hide();
