@@ -3,7 +3,7 @@
 *       D o u b l e - P r e c i s i o n   4 - E l e m e n t   V e c t o r       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1994,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1994,2006 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXVec4d.cpp,v 1.9 2005/01/16 16:06:07 fox Exp $                          *
+* $Id: FXVec4d.cpp,v 1.15 2006/01/22 17:58:51 fox Exp $                         *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -30,6 +30,9 @@
 #include "FXVec2d.h"
 #include "FXVec3d.h"
 #include "FXVec4d.h"
+#include "FXQuatd.h"
+#include "FXMat4d.h"
+
 
 using namespace FX;
 
@@ -59,9 +62,9 @@ FXVec4d::operator FXColor() const {
   }
 
 
-FXVec4d normalize(const FXVec4d& a){
-  register FXdouble t=len(a);
-  if(t>0.0){ return FXVec4d(a.x/t,a.y/t,a.z/t,a.w/t); }
+FXVec4d normalize(const FXVec4d& v){
+  register FXdouble t=v.length();
+  if(t>0.0){ return FXVec4d(v.x/t,v.y/t,v.z/t,v.w/t); }
   return FXVec4d(0.0,0.0,0.0,0.0);
   }
 
@@ -95,14 +98,20 @@ FXVec4d plane(const FXVec4d& vec){
 
 
 // Signed distance normalized plane and point
-FXdouble distance(const FXVec4d& plane,const FXVec3d& p){
-  return plane.x*p.x+plane.y*p.y+plane.z*p.z+plane.z;
+FXdouble FXVec4d::distance(const FXVec3d& p) const {
+  return x*p.x+y*p.y+z*p.z+w;
   }
 
 
 // Return true if edge a-b crosses plane
-FXbool crosses(const FXVec4d& plane,const FXVec3d& a,const FXVec3d& b){
-  return (distance(plane,a)>=0.0) ^ (distance(plane,b)>=0.0);
+bool FXVec4d::crosses(const FXVec3d& a,const FXVec3d& b) const {
+  return (distance(a)>=0.0) ^ (distance(b)>=0.0);
+  }
+
+
+// Vector times matrix
+FXVec4d FXVec4d::operator*(const FXMat4d& m) const {
+  return FXVec4d(x*m[0][0]+y*m[1][0]+z*m[2][0]+w*m[3][0], x*m[0][1]+y*m[1][1]+z*m[2][1]+w*m[3][1], x*m[0][2]+y*m[1][2]+z*m[2][2]+w*m[3][2], x*m[0][3]+y*m[1][3]+z*m[2][3]+w*m[3][3]);
   }
 
 
